'use strict';

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _slicedToArray = require('babel-runtime/helpers/sliced-to-array')['default'];

var _toConsumableArray = require('babel-runtime/helpers/to-consumable-array')['default'];

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

var _regeneratorRuntime = require('babel-runtime/regenerator')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});
var isIPv6 = require('net').isIPv6;
var isIPv4 = require('net').isIPv4;

// TODO: hostname needs to only accept valid hosts / ipv4 / [ipv6] , YOLO
var URLPattern = /^(?:(([a-zA-Z\-\.\+]+:)(?:\/\/)?|(?:\/\/))(?:([^:]+)?(?:[:]([^@]*))?@)?([^:\/\?\#]+)?(?:[:](\d+)(?=[\/?#]|$))?)?([^?#]*)?([?][^#]*)?([#][\s\S]*)?$/;

function stringifyURL(protocol, auth, host, port, pathname, search, hash) {
  var ret = '';
  // complex parts cannot be strings
  if (auth instanceof URLAuth === false) throw TypeError('Expected a URLAuth');
  if (pathname instanceof URLPathname === false) throw TypeError('Expected a URLPathname');
  if (search instanceof URLSearch === false) throw TypeError('Expected a URLSearch');
  if (protocol !== undefined) ret += protocol + '//';
  // protocol relative URL
  else if (auth.empty() === false || host !== undefined || port !== undefined) {
      ret += '//';
    }
  ret += auth;
  if (host !== undefined) ret += host;
  if (port !== undefined) ret += ':' + port;
  if (!pathname.empty()) ret += pathname.raw();
  ret += search;
  if (hash !== undefined) ret += hash;
  return ret;
}

var EMPTY_URL = new Array(9);

var ComposableURL = (function () {
  function ComposableURL(str) {
    _classCallCheck(this, ComposableURL);

    var parts = str === undefined ? EMPTY_URL : URLPattern.exec(str);

    var _parts = _slicedToArray(parts, 10);

    var specified_protocol = _parts[1];
    var protocol = _parts[2];
    var user = _parts[3];
    var secret = _parts[4];
    var hostname = _parts[5];
    var port = _parts[6];
    var pathname = _parts[7];
    var search = _parts[8];
    var hash = _parts[9];

    if (user !== undefined) {
      user = decodeURIComponent(user);
    }
    if (secret !== undefined) {
      secret = decodeURIComponent(secret);
    }
    if (pathname !== undefined) {
      pathname = decodeURIComponent(pathname);
    }
    if (search !== undefined) {
      search = decodeURIComponent(search);
    }
    if (hash !== undefined) {
      hash = decodeURIComponent(hash);
    }
    if (hostname !== undefined) {
      if (hostname[0] === '[') {
        if (hostname[hostname.length - 1] !== ']') {
          throw new Error('unmatched [');
        }
        if (isIPv6(hostname.slice(1, -1) === false)) {
          throw new Error('expected IPv6 address inside [...]');
        }
      }
    }
    this.protocol = protocol;

    this.auth = new URLAuth(user, secret);
    this.hostname = hostname;
    this.port = port === undefined ? undefined : parseInt(port, 10);
    this.pathname = new URLPathname(pathname);
    this.search = new URLSearch(search);
    this.hash = hash;

    this.isProtocolRelative = Boolean(protocol === undefined && specified_protocol === '//');
  }

  _createClass(ComposableURL, [{
    key: 'resolve',
    value: function resolve(str) {
      var url = undefined;
      if (typeof str === 'string') url = new ComposableURL(str);else url = str;

      var ret = this;
      if (url.protocol !== undefined) {
        if (this.isProtocolRelative) {
          if (url.protocol !== undefined) {
            ret = ret.withProtocol(url.protocol);
          }
        } else if (this.protocol === undefined) {
          if (url.protocol !== undefined) {
            throw new Error('Cannot resolve a scheme-less URL with a schemed URL');
          }
          // neither have protocols, ignore
        } else {
            ret = ret.withProtocol(url.protocol);
          }
      }
      if (url.auth.empty() === false) {
        ret = ret.withAuth(url.auth);
      }
      if (url.hostname !== undefined) {
        ret = ret.withHostname(url.hostname);
      }
      if (url.port !== undefined) {
        ret = ret.withPort(url.port);
      }
      if (url.pathname !== undefined && url.pathname.empty() === false) {
        var raw = url.pathname.raw();
        // absolute path
        if (raw.length > 0 && raw[0] === '/') {
          ret = ret.withPathname(url.pathname);
        } else {
          if ((ret.isProtocolRelative || ret.protocol !== undefined) && ret.pathname.empty()) {
            ret = ret.withPathname(ret.pathname.concat('', raw));
          } else {
            ret = ret.withPathname(ret.pathname.concat(raw));
          }
        }
      }
      if (url.search.empty() === false) {
        ret = ret.withSearch(url.search);
      }
      if (url.hash !== undefined) {
        ret = ret.withHash(url.hash);
      }
      return ret;
    }
  }, {
    key: 'withProtocol',
    value: function withProtocol(protocol) {
      return new ComposableURL(stringifyURL(protocol, this.auth, this.hostname, this.port, this.pathname, this.search, this.hash));
    }
  }, {
    key: 'withAuth',
    value: function withAuth(auth) {
      return new ComposableURL(stringifyURL(this.protocol, auth, this.hostname, this.port, this.pathname, this.search, this.hash));
    }
  }, {
    key: 'withHostname',
    value: function withHostname(hostname) {
      return new ComposableURL(stringifyURL(this.protocol, this.auth, hostname, this.port, this.pathname, this.search, this.hash));
    }
  }, {
    key: 'withPort',
    value: function withPort(port) {
      return new ComposableURL(stringifyURL(this.protocol, this.auth, this.hostname, port, this.pathname, this.search, this.hash));
    }
  }, {
    key: 'withPathname',
    value: function withPathname(pathname) {
      return new ComposableURL(stringifyURL(this.protocol, this.auth, this.hostname, this.port, typeof pathname === 'string' ? new URLPathname(pathname) : pathname, this.search, this.hash));
    }
  }, {
    key: 'withSearch',
    value: function withSearch(search) {
      return new ComposableURL(stringifyURL(this.protocol, this.auth, this.hostname, this.port, this.pathname, typeof search === 'string' ? new URLSearch(search) : search, this.hash));
    }
  }, {
    key: 'withHash',
    value: function withHash(hash) {
      return new ComposableURL(stringifyURL(this.protocol, this.auth, this.hostname, this.port, this.pathname, this.search, '#' + hash));
    }
  }, {
    key: 'toString',
    value: function toString(see_auth) {
      return stringifyURL(this.protocol, see_auth === true ? this.auth : new URLAuth(), this.hostname, this.port, this.pathname, this.search, this.hash);
    }
  }]);

  return ComposableURL;
})();

exports.ComposableURL = ComposableURL;

var URLAuth = (function () {
  function URLAuth(user, secret) {
    _classCallCheck(this, URLAuth);

    this.user = user;
    this.secret = secret;
  }

  _createClass(URLAuth, [{
    key: 'empty',
    value: function empty() {
      if (this.user === undefined && this.secret === undefined) return true;
      return false;
    }
  }, {
    key: 'toString',
    value: function toString() {
      if (this.empty()) return '';
      var user = this.user === undefined ? '' : encodeURIComponent(this.user);
      if (this.secret !== undefined) {
        var secret = encodeURIComponent(this.secret);
        return user + ':' + secret + '@';
      } else {
        return user + '@';
      }
    }
  }, {
    key: 'inspect',
    value: function inspect() {
      if (this.user === undefined && this.secret === undefined) return '';
      var user = this.user === undefined ? '' : encodeURIComponent(this.user);
      if (this.secret !== undefined) {
        var secret = encodeURIComponent(this.secret);
        return user + ':' + secret + '@';
      } else {
        return user + '@';
      }
    }
  }]);

  return URLAuth;
})();

exports.URLAuth = URLAuth;

var URLSearch = (function () {
  function URLSearch(str) {
    _classCallCheck(this, URLSearch);

    this._entries = [];
    if (str === undefined) {
      return;
    }
    if (str[0] === '?') {
      str = str.slice(1);
    }
    if (str.length === 0) return;
    var parts = str.split('&');
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = _getIterator(parts), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var pair = _step.value;

        if (pair === '') continue;
        var eq = pair.indexOf('=');
        if (eq === -1) {
          this._entries[this._entries.length] = {
            key: decodeURIComponent(pair),
            value: ''
          };
        } else {
          this._entries[this._entries.length] = {
            key: decodeURIComponent(pair.slice(0, eq)),
            value: decodeURIComponent(pair.slice(eq + 1))
          };
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator['return']) {
          _iterator['return']();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  }

  _createClass(URLSearch, [{
    key: 'empty',
    value: function empty() {
      return this._entries.length === 0;
    }
  }, {
    key: 'entries',
    value: _regeneratorRuntime.mark(function entries(key) {
      var _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2, entry;

      return _regeneratorRuntime.wrap(function entries$(context$2$0) {
        while (1) switch (context$2$0.prev = context$2$0.next) {
          case 0:
            _iteratorNormalCompletion2 = true;
            _didIteratorError2 = false;
            _iteratorError2 = undefined;
            context$2$0.prev = 3;
            _iterator2 = _getIterator(this._entries);

          case 5:
            if (_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done) {
              context$2$0.next = 12;
              break;
            }

            entry = _step2.value;
            context$2$0.next = 9;
            return entry;

          case 9:
            _iteratorNormalCompletion2 = true;
            context$2$0.next = 5;
            break;

          case 12:
            context$2$0.next = 18;
            break;

          case 14:
            context$2$0.prev = 14;
            context$2$0.t0 = context$2$0['catch'](3);
            _didIteratorError2 = true;
            _iteratorError2 = context$2$0.t0;

          case 18:
            context$2$0.prev = 18;
            context$2$0.prev = 19;

            if (!_iteratorNormalCompletion2 && _iterator2['return']) {
              _iterator2['return']();
            }

          case 21:
            context$2$0.prev = 21;

            if (!_didIteratorError2) {
              context$2$0.next = 24;
              break;
            }

            throw _iteratorError2;

          case 24:
            return context$2$0.finish(21);

          case 25:
            return context$2$0.finish(18);

          case 26:
          case 'end':
            return context$2$0.stop();
        }
      }, entries, this, [[3, 14, 18, 26], [19,, 21, 25]]);
    })
  }, {
    key: 'concat',
    value: function concat(_ref) {
      var key = _ref.key;
      var value = _ref.value;

      return new URLSearch([].concat(_toConsumableArray(this.entries), [{ key: key, value: value }]));
    }
  }, {
    key: 'toString',
    value: function toString() {
      var entries = this.entries();

      var _entries$next = entries.next();

      var done = _entries$next.done;
      var value = _entries$next.value;

      if (done) return '';else {
        var ret = '?';
        ret += encodeURIComponent(value.key) + '=' + encodeURIComponent(value.value);
        var _iteratorNormalCompletion3 = true;
        var _didIteratorError3 = false;
        var _iteratorError3 = undefined;

        try {
          for (var _iterator3 = _getIterator(entries), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
            var _step3$value = _step3.value;
            var key = _step3$value.key;
            var _value = _step3$value.value;

            ret += '&' + encodeURIComponent(key) + '=' + encodeURIComponent(_value);
          }
        } catch (err) {
          _didIteratorError3 = true;
          _iteratorError3 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion3 && _iterator3['return']) {
              _iterator3['return']();
            }
          } finally {
            if (_didIteratorError3) {
              throw _iteratorError3;
            }
          }
        }

        return ret;
      }
    }
  }, {
    key: 'inspect',
    value: function inspect() {
      return this.toString();
    }
  }]);

  return URLSearch;
})();

exports.URLSearch = URLSearch;

var URLPathname = (function () {
  function URLPathname(str_or_arr) {
    _classCallCheck(this, URLPathname);

    var parts = undefined;
    if (typeof str_or_arr === 'undefined') {
      parts = [];
    } else if (typeof str_or_arr === 'string') {
      parts = str_or_arr.split('/');
    } else if (Array.isArray(str_or_arr)) {
      parts = str_or_arr;
    } else {
      throw TypeError('Expected a String or Array');
    }
    var _iteratorNormalCompletion4 = true;
    var _didIteratorError4 = false;
    var _iteratorError4 = undefined;

    try {
      for (var _iterator4 = _getIterator(parts), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
        var item = _step4.value;

        if (typeof item !== 'string') {
          throw TypeError('Expected a String');
        }
      }
    } catch (err) {
      _didIteratorError4 = true;
      _iteratorError4 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion4 && _iterator4['return']) {
          _iterator4['return']();
        }
      } finally {
        if (_didIteratorError4) {
          throw _iteratorError4;
        }
      }
    }

    this._items = [].concat(_toConsumableArray(parts));
    this._notSplit = true;
    this._parts = null;
    this._notNormalized = true;
    this._normal = null;
  }

  _createClass(URLPathname, [{
    key: 'empty',
    value: function empty() {
      if (this._notSplit) return this._items.length === 0;
      return this._parts.length === 0;
    }
  }, {
    key: 'concat',
    value: function concat() {
      var parts = this._notSplit ? this._items : this._parts;
      var ret = [].concat(_toConsumableArray(parts));
      var _iteratorNormalCompletion5 = true;
      var _didIteratorError5 = false;
      var _iteratorError5 = undefined;

      try {
        for (var _len = arguments.length, str = Array(_len), _key = 0; _key < _len; _key++) {
          str[_key] = arguments[_key];
        }

        for (var _iterator5 = _getIterator(str), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
          var path = _step5.value;

          if (typeof path !== 'string') throw new Error('Expected a String');
          if (path[0] === '/') {
            ret.length = 0;
          }
          ret[ret.length] = path;
        }
      } catch (err) {
        _didIteratorError5 = true;
        _iteratorError5 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion5 && _iterator5['return']) {
            _iterator5['return']();
          }
        } finally {
          if (_didIteratorError5) {
            throw _iteratorError5;
          }
        }
      }

      var new_url = new URLPathname(ret);
      return new_url;
    }
  }, {
    key: 'entries',
    value: _regeneratorRuntime.mark(function entries() {
      var _iteratorNormalCompletion6, _didIteratorError6, _iteratorError6, _iterator6, _step6, item;

      return _regeneratorRuntime.wrap(function entries$(context$2$0) {
        while (1) switch (context$2$0.prev = context$2$0.next) {
          case 0:
            if (this._notSplit) {
              if (this._items.length > 0) {
                this._parts = this._items.join('/').split('/');
              } else {
                this._parts = [];
              }
              this._notSplit = false;
              this._items = null;
            }
            _iteratorNormalCompletion6 = true;
            _didIteratorError6 = false;
            _iteratorError6 = undefined;
            context$2$0.prev = 4;
            _iterator6 = _getIterator(this._parts);

          case 6:
            if (_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done) {
              context$2$0.next = 13;
              break;
            }

            item = _step6.value;
            context$2$0.next = 10;
            return item;

          case 10:
            _iteratorNormalCompletion6 = true;
            context$2$0.next = 6;
            break;

          case 13:
            context$2$0.next = 19;
            break;

          case 15:
            context$2$0.prev = 15;
            context$2$0.t0 = context$2$0['catch'](4);
            _didIteratorError6 = true;
            _iteratorError6 = context$2$0.t0;

          case 19:
            context$2$0.prev = 19;
            context$2$0.prev = 20;

            if (!_iteratorNormalCompletion6 && _iterator6['return']) {
              _iterator6['return']();
            }

          case 22:
            context$2$0.prev = 22;

            if (!_didIteratorError6) {
              context$2$0.next = 25;
              break;
            }

            throw _iteratorError6;

          case 25:
            return context$2$0.finish(22);

          case 26:
            return context$2$0.finish(19);

          case 27:
          case 'end':
            return context$2$0.stop();
        }
      }, entries, this, [[4, 15, 19, 27], [20,, 22, 26]]);
    })
  }, {
    key: 'raw',
    value: function raw() {
      var entries = this.entries();

      var _entries$next2 = entries.next();

      var done = _entries$next2.done;
      var value = _entries$next2.value;

      if (done) {
        return '';
      } else {
        var ret = [value];
        var _iteratorNormalCompletion7 = true;
        var _didIteratorError7 = false;
        var _iteratorError7 = undefined;

        try {
          for (var _iterator7 = _getIterator(entries), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
            var item = _step7.value;

            ret[ret.length] = item;
          }
        } catch (err) {
          _didIteratorError7 = true;
          _iteratorError7 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion7 && _iterator7['return']) {
              _iterator7['return']();
            }
          } finally {
            if (_didIteratorError7) {
              throw _iteratorError7;
            }
          }
        }

        return ret.join('/');
      }
    }
  }, {
    key: 'normalize',
    value: function normalize() {
      if (this._notNormalized) {
        var entries = this.entries();

        var _entries$next3 = entries.next();

        var done = _entries$next3.done;
        var value = _entries$next3.value;

        if (done) {
          this._normal = '';
        } else {
          var ret = [value];
          var _iteratorNormalCompletion8 = true;
          var _didIteratorError8 = false;
          var _iteratorError8 = undefined;

          try {
            for (var _iterator8 = _getIterator(entries), _step8; !(_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done); _iteratorNormalCompletion8 = true) {
              var item = _step8.value;

              var prev = ret[ret.length - 1];
              if (item == '' || item == '.') {
                continue;
              } else if (item == '..' && ret.length > 0 && prev !== '..') {
                if (prev === '.') {
                  ret[ret.length - 1] = '..';
                } else {
                  ret.length -= 1;
                }
              } else {
                ret[ret.length] = item;
              }
            }
          } catch (err) {
            _didIteratorError8 = true;
            _iteratorError8 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion8 && _iterator8['return']) {
                _iterator8['return']();
              }
            } finally {
              if (_didIteratorError8) {
                throw _iteratorError8;
              }
            }
          }

          this._normal = ret.join('/');
          if (this._normal === '') this._normal = '/';
        }
        this._notNormalized = false;
      }
      return this._normal;
    }

    // convert this path to a base path
    // this will remove *relative* pathing
  }, {
    key: 'base',
    value: function base() {
      return this.raw().replace(/^\/?(?:\.\.?(?:\/|$))+|^\//, '');
    }
  }, {
    key: 'safer',
    value: function safer() {
      return new URLPathname(this.normalize()).base();
    }
  }, {
    key: 'toString',
    value: function toString() {
      return this.raw();
    }
  }, {
    key: 'inspect',
    value: function inspect() {
      return this.raw();
    }
  }]);

  return URLPathname;
})();

exports.URLPathname = URLPathname;
//# sourceMappingURL=index.js.map