'use strict';

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

var _interopRequireWildcard = require('babel-runtime/helpers/interop-require-wildcard')['default'];

var _ = require('./');

var _assert = require('assert');

var assert = _interopRequireWildcard(_assert);

function check(_ref) {
  var url = _ref.url;
  var protocol = _ref.protocol;
  var auth = _ref.auth;
  var hostname = _ref.hostname;
  var port = _ref.port;
  var pathname = _ref.pathname;
  var search = _ref.search;
  var hash = _ref.hash;
  var isProtocolRelative = _ref.isProtocolRelative;
  var string = _ref.string;
  var pathname_tests = _ref.pathname_tests;

  if (auth === undefined) {
    auth = new _.URLAuth();
  }
  if (pathname === undefined) {
    pathname = new _.URLPathname();
  } else {
    pathname = new _.URLPathname(pathname.raw());
  }
  if (search === undefined) {
    search = new _.URLSearch();
  }
  var fixture = {
    protocol: protocol,
    auth: auth,
    hostname: hostname,
    port: port,
    pathname: pathname,
    search: search,
    hash: hash,
    isProtocolRelative: Boolean(isProtocolRelative)
  };
  assert.deepEqual(url, fixture);
  var test_str = String(url);
  assert.equal(test_str, string);
  var auth_str = String(auth);
  if (auth_str) {
    var authed_str = url.toString(true);
    assert.notEqual(authed_str, test_str);
    assert.equal(url.toString(true).replace(auth_str, ''), test_str);
  }
  if (pathname_tests) {
    var normalize = pathname_tests.normalize;
    var base = pathname_tests.base;
    var safer = pathname_tests.safer;

    if (base) assert.equal(base, url.pathname.base());
    if (normalize) assert.equal(normalize, url.pathname.normalize());
    if (safer) assert.equal(safer, url.pathname.safer());
  }
}
var checks = [{
  url: new _.ComposableURL('http://google.com').resolve('/..'),
  protocol: 'http:',
  hostname: 'google.com',
  pathname: new _.URLPathname(['', '..']),
  string: 'http://google.com/..',
  pathname_tests: {
    normalize: '/',
    base: '',
    safer: ''
  }
}, {
  url: new _.ComposableURL('http://google.com/t').resolve('/..'),
  protocol: 'http:',
  hostname: 'google.com',
  pathname: new _.URLPathname(['/..']),
  string: 'http://google.com/..',
  pathname_tests: {
    normalize: '/',
    base: '',
    safer: ''
  }
}, {
  url: new _.ComposableURL('node_modules'),
  pathname: new _.URLPathname(['node_modules']),
  string: 'node_modules',
  pathname_tests: {
    normalize: 'node_modules',
    base: 'node_modules',
    safer: 'node_modules'
  }
}, {
  url: new _.ComposableURL('node_modules/t'),
  pathname: new _.URLPathname(['node_modules', 't']),
  string: 'node_modules/t'
}, {
  url: new _.ComposableURL('./node_modules/t'),
  pathname: new _.URLPathname(['.', 'node_modules', 't']),
  string: './node_modules/t',
  pathname_tests: {
    normalize: './node_modules/t',
    base: 'node_modules/t',
    safer: 'node_modules/t'
  }
}, {
  url: new _.ComposableURL('./node_modules/../t'),
  pathname: new _.URLPathname(['.', 'node_modules', '..', 't']),
  string: './node_modules/../t',
  pathname_tests: {
    normalize: './t',
    base: 'node_modules/../t',
    safer: 't'
  }
}, {
  url: new _.ComposableURL('//./node_modules/../t'),
  hostname: '.',
  pathname: new _.URLPathname(['', 'node_modules', '..', 't']),
  isProtocolRelative: true,
  string: '//./node_modules/../t',
  pathname_tests: {
    normalize: '/t',
    base: 'node_modules/../t',
    safer: 't'
  }
}, {
  url: new _.ComposableURL('//goog.ly'),
  hostname: 'goog.ly',
  isProtocolRelative: true,
  string: '//goog.ly'
}, {
  url: new _.ComposableURL('//goog.ly:1337'),
  hostname: 'goog.ly',
  port: 1337,
  isProtocolRelative: true,
  string: '//goog.ly:1337'
}, {
  url: new _.ComposableURL('//goog.ly:01337'),
  hostname: 'goog.ly',
  port: 1337,
  isProtocolRelative: true,
  string: '//goog.ly:1337'
}, {
  url: new _.ComposableURL('//:123@goog.ly'),
  auth: new _.URLAuth(undefined, '123'),
  hostname: 'goog.ly',
  isProtocolRelative: true,
  string: '//goog.ly'
}, {
  url: new _.ComposableURL('//bmeck:123@goog.ly'),
  auth: new _.URLAuth('bmeck', '123'),
  hostname: 'goog.ly',
  isProtocolRelative: true,
  string: '//goog.ly'
}, {
  url: new _.ComposableURL('//bmeck@goog.ly'),
  auth: new _.URLAuth('bmeck', undefined),
  hostname: 'goog.ly',
  isProtocolRelative: true,
  string: '//goog.ly'
}];
var _iteratorNormalCompletion = true;
var _didIteratorError = false;
var _iteratorError = undefined;

try {
  for (var _iterator = _getIterator(checks), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
    var item = _step.value;

    check(item);
  }
} catch (err) {
  _didIteratorError = true;
  _iteratorError = err;
} finally {
  try {
    if (!_iteratorNormalCompletion && _iterator['return']) {
      _iterator['return']();
    }
  } finally {
    if (_didIteratorError) {
      throw _iteratorError;
    }
  }
}

assert.deepEqual(new _.ComposableURL('cat').resolve('dog'), new _.ComposableURL('cat/dog'));
assert.deepEqual(new _.ComposableURL('http:').resolve('//google.com'), new _.ComposableURL('http://google.com'));
assert.deepEqual(new _.ComposableURL('http:').resolve('google.com'), new _.ComposableURL('http:///google.com'));
assert.deepEqual(new _.ComposableURL('http:').resolve('google.com'), new _.ComposableURL('http:/google.com'));
;{
  var url = new _.ComposableURL('http://google.com/t').resolve('./..');
  url = url.withPathname(url.pathname.normalize());
  assert.deepEqual(url, new _.ComposableURL('http://google.com/'));
}
;{
  var url = new _.ComposableURL('http://google.com/t').resolve('./');
  url = url.withPathname(url.pathname.normalize());
  assert.deepEqual(url, new _.ComposableURL('http://google.com/t'));
}
;{
  var url = new _.ComposableURL('http://google.com/t').resolve('.');
  url = url.withPathname(url.pathname.normalize());
  assert.deepEqual(url, new _.ComposableURL('http://google.com/t'));
}
//# sourceMappingURL=test.js.map